/**
 * @file
 * Free/busy data for specified users
 *
 * @addtogroup libfreebusy
 * @{
 */

#ifndef ECFREEBUSYSUPPORT_H
#define ECFREEBUSYSUPPORT_H

#include "freebusy.h"
#include "freebusyguid.h"

#include <mapi.h>
#include <mapidefs.h>
#include <mapix.h>

#include "freebusy.h"
#include "freebusyguid.h"

#include "ECUnknown.h"
#include "Trace.h"
#include "ECDebug.h"
#include "ECGuid.h"


#include "ECFBBlockList.h"

/**
 * Implementatie of the IFreeBusySupport interface
 */
class ECFreeBusySupport : public ECUnknown
{
private:
	ECFreeBusySupport(void);
	~ECFreeBusySupport(void);
public:
	static HRESULT Create(ECFreeBusySupport** lppFreeBusySupport);

	// From IUnknown
		virtual HRESULT QueryInterface(REFIID refiid, void **lppInterface);

		// IFreeBusySupport
		virtual HRESULT Open(IMAPISession* lpMAPISession, IMsgStore* lpMsgStore, BOOL bStore);
		virtual HRESULT Close();
		virtual HRESULT LoadFreeBusyData(	ULONG cMax, FBUser *rgfbuser, IFreeBusyData **prgfbdata,
											HRESULT *phrStatus, ULONG *pcRead);

		virtual HRESULT LoadFreeBusyUpdate(ULONG cUsers, FBUser *lpUsers, IFreeBusyUpdate **lppFBUpdate, ULONG *lpcFBUpdate, void *lpData4);
		virtual HRESULT CommitChanges();
		virtual HRESULT GetDelegateInfo(FBUser, void *);
		virtual HRESULT SetDelegateInfo(void *);
		virtual HRESULT AdviseFreeBusy(void *);
		virtual HRESULT Reload(void *);
		virtual HRESULT GetFBDetailSupport(void **, BOOL );
		virtual HRESULT HrHandleServerSched(void *);
		virtual HRESULT HrHandleServerSchedAccess();
		virtual BOOL FShowServerSched(BOOL );
		virtual HRESULT HrDeleteServerSched();
		virtual HRESULT GetFReadOnly(void *);
		virtual HRESULT SetLocalFB(void *);
		virtual HRESULT PrepareForSync();
		virtual HRESULT GetFBPublishMonthRange(void *);
		virtual HRESULT PublishRangeChanged();
		virtual HRESULT CleanTombstone();
		virtual HRESULT GetDelegateInfoEx(FBUser sFBUser, unsigned int *lpulStatus, unsigned int *lpulStart, unsigned int *lpulEnd);
		virtual HRESULT PushDelegateInfoToWorkspace();
		virtual HRESULT Placeholder21(void *, HWND, BOOL );
		virtual HRESULT Placeholder22();

public:
	// Interface voor Outlook 2002 and up
	class xFreeBusySupport : public IFreeBusySupport
	{
		public:
			// From IUnknown
			virtual HRESULT __stdcall QueryInterface(REFIID refiid, void **lppInterface);
			virtual ULONG __stdcall AddRef();
			virtual ULONG __stdcall Release();

			// From IFreeBusySupport
			virtual HRESULT __stdcall Open(IMAPISession* lpMAPISession, IMsgStore* lpMsgStore, BOOL bStore);
			virtual HRESULT __stdcall Close();
			virtual HRESULT __stdcall LoadFreeBusyData(	ULONG cMax, FBUser *rgfbuser, IFreeBusyData **prgfbdata,
												HRESULT *phrStatus, ULONG *pcRead);

			virtual HRESULT __stdcall LoadFreeBusyUpdate(ULONG cUsers, FBUser *lpUsers, IFreeBusyUpdate **lppFBUpdate, ULONG *lpcFBUpdate, void *lpData4);
			virtual HRESULT __stdcall CommitChanges();
			virtual HRESULT __stdcall GetDelegateInfo(FBUser fbUser, void *lpData);
			virtual HRESULT __stdcall SetDelegateInfo(void *lpData);
			virtual HRESULT __stdcall AdviseFreeBusy(void *lpData);
			virtual HRESULT __stdcall Reload(void *lpData);
			virtual HRESULT __stdcall GetFBDetailSupport(void **lppData, BOOL bData);
			virtual HRESULT __stdcall HrHandleServerSched(void *lpData);
			virtual HRESULT __stdcall HrHandleServerSchedAccess();
			virtual BOOL __stdcall FShowServerSched(BOOL bData);
			virtual HRESULT __stdcall HrDeleteServerSched();
			virtual HRESULT __stdcall GetFReadOnly(void *lpData);
			virtual HRESULT __stdcall SetLocalFB(void *lpData);
			virtual HRESULT __stdcall PrepareForSync();
			virtual HRESULT __stdcall GetFBPublishMonthRange(void *lpData);
			virtual HRESULT __stdcall PublishRangeChanged();
			virtual HRESULT __stdcall CleanTombstone();
			virtual HRESULT __stdcall GetDelegateInfoEx(FBUser fbUser, unsigned int *lpData1, unsigned int *lpData2, unsigned int *lpData3);
			virtual HRESULT __stdcall PushDelegateInfoToWorkspace();
			virtual HRESULT __stdcall Placeholder21(void *lpData, HWND hwnd, BOOL bData);
			virtual HRESULT __stdcall Placeholder22();
	} m_xFreeBusySupport;

	// Interface for Outlook 2000
	class xFreeBusySupportOutlook2000 : public IFreeBusySupportOutlook2000
	{
		public:
			// From IUnknown
			virtual HRESULT __stdcall QueryInterface(REFIID refiid, void **lppInterface);
			virtual ULONG __stdcall AddRef();
			virtual ULONG __stdcall Release();

			// From IFreeBusySupport
			virtual HRESULT __stdcall Open(IMAPISession* lpMAPISession, IMsgStore* lpMsgStore, BOOL bStore);
			virtual HRESULT __stdcall Close();
			virtual HRESULT __stdcall LoadFreeBusyData(	ULONG cMax, FBUser *rgfbuser, IFreeBusyData **prgfbdata,
												HRESULT *phrStatus, ULONG *pcRead);

			virtual HRESULT __stdcall LoadFreeBusyUpdate(ULONG cUsers, FBUser *lpUsers, IFreeBusyUpdate **lppFBUpdate, ULONG *lpcFBUpdate, void *lpData4);
			virtual HRESULT __stdcall CommitChanges();
			virtual HRESULT __stdcall GetDelegateInfo(FBUser fbUser, void *lpData);
			virtual HRESULT __stdcall SetDelegateInfo(void *lpData);
			virtual HRESULT __stdcall AdviseFreeBusy(void *lpData);
			virtual HRESULT __stdcall Reload(void *lpData);
			virtual HRESULT __stdcall GetFBDetailSupport(void **lppData, BOOL bData);
			virtual HRESULT __stdcall HrHandleServerSched(void *lpData);
			virtual HRESULT __stdcall HrHandleServerSchedAccess();
			virtual BOOL __stdcall FShowServerSched(BOOL bData);
			virtual HRESULT __stdcall HrDeleteServerSched();
			virtual HRESULT __stdcall GetFReadOnly(void *lpData);
			virtual HRESULT __stdcall SetLocalFB(void *lpData);
			virtual HRESULT __stdcall PrepareForSync();
			virtual HRESULT __stdcall GetFBPublishMonthRange(void *lpData);
			virtual HRESULT __stdcall PublishRangeChanged();
			//virtual HRESULT __stdcall CleanTombstone();
			virtual HRESULT __stdcall GetDelegateInfoEx(FBUser fbUser, unsigned int *lpData1, unsigned int *lpData2, unsigned int *lpData3);
			virtual HRESULT __stdcall PushDelegateInfoToWorkspace();
			virtual HRESULT __stdcall Placeholder21(void *lpData, HWND hwnd, BOOL bData);
			virtual HRESULT __stdcall Placeholder22();
	} m_xFreeBusySupportOutlook2000;

private:
	IMAPISession*	m_lpSession;
	IMsgStore*		m_lpPublicStore;
	IMsgStore*		m_lpUserStore;
	IMAPIFolder*	m_lpFreeBusyFolder;
	unsigned int	m_ulOutlookVersion;
};

#endif // ECFREEBUSYSUPPORT_H

/** @} */
