#ifndef ECFIFOBUFFER_H
#define ECFIFOBUFFER_H

#include <deque>
#include <pthread.h>

#include "ZarafaCode.h"

// Thread safe buffer for FIFO operations
class ECFifoBuffer
{
public:
	typedef std::deque<unsigned char>	storage_type;
	typedef storage_type::size_type		size_type;
	enum close_flags { cfRead = 1, cfWrite = 2 };

public:
	ECFifoBuffer(size_type ulMaxSize = 131072);
	~ECFifoBuffer();
	
	ECRESULT Write(const void *lpBuf, size_type cbBuf, unsigned int ulTimeoutMs, size_type *lpcbWritten);
	ECRESULT Read(void *lpBuf, size_type cbBuf, unsigned int ulTimeoutMs, size_type *lpcbRead);
	ECRESULT Close(close_flags flags);
	ECRESULT Flush();

	bool IsClosed(ULONG flags) const;
	bool IsEmpty() const;
	bool IsFull() const;
	unsigned long Size();
	
private:
	// prohibit copy
	ECFifoBuffer(const ECFifoBuffer &);
	ECFifoBuffer& operator=(const ECFifoBuffer &);
	
private:
	storage_type	m_storage;
	size_type		m_ulMaxSize;
	bool			m_bReaderClosed;
	bool            m_bWriterClosed;

	pthread_mutex_t	m_hMutex;
	pthread_cond_t	m_hCondNotEmpty;
	pthread_cond_t	m_hCondNotFull;
	pthread_cond_t	m_hCondFlushed;
};


// inlines
inline bool ECFifoBuffer::IsClosed(ULONG flags) const {
	switch (flags) {
	case cfRead:
		return m_bReaderClosed;
	case cfWrite:
		return m_bWriterClosed;
	case cfRead|cfWrite:
		return m_bReaderClosed && m_bWriterClosed;
	default:
		ASSERT(FALSE);
		return false;
	}
}

inline bool ECFifoBuffer::IsEmpty() const {
	return m_storage.empty();
}

inline bool ECFifoBuffer::IsFull() const {
	return m_storage.size() == m_ulMaxSize;
}

inline unsigned long ECFifoBuffer::Size() {
	return m_storage.size();
}

#endif // ndef ECFIFOBUFFER_H
