#ifndef ECSCHEDULER_H
#define ECSCHEDULER_H

#include "ECLogger.h"

#include <pthread.h>
#include <list>

enum eSchedulerType{
	SCHEDULE_NONE,
	SCHEDULE_SECONDS,
	SCHEDULE_MINUTES,
	SCHEDULE_HOUR,
	SCHEDULE_DAY,
	SCHEDULE_MONTH
};

typedef struct tagSchedule
{
	eSchedulerType	eType;
	unsigned int	ulBeginCycle;
	time_t			tLastRunTime;
	void*			(*lpFunction)(void*);
	void*			lpData;
}ECSCHEDULE, *LPECSCHEDULE;

typedef std::list<ECSCHEDULE> ECScheduleList;

class ECScheduler
{
public:
	ECScheduler(ECLogger *lpLogger);
	~ECScheduler(void);

	HRESULT AddSchedule(eSchedulerType eType, unsigned int ulBeginCycle, void* (*lpFunction)(void*), void* lpData = NULL);

private:
	static bool hasExpired(time_t ttime, LPECSCHEDULE lpSchedule);
	static void* ScheduleThread(void* lpTmpScheduler);

private:
	ECScheduleList		m_listScheduler;
	ECLogger *			m_lpLogger;

	bool				m_bExit;
	pthread_mutex_t     m_hExitMutex;			// Mutex needed for the release signal
	pthread_cond_t		m_hExitSignal;			// Signal that should be send to the Scheduler when to exit 
	pthread_mutex_t		m_hSchedulerMutex;		// Mutex for the locking of the Scheduler
	pthread_t			m_hMainThread;			// Thread that is used for the Scheduler
};

#endif
