#ifndef ECSERIALIZER_H
#define ECSERIALIZER_H

#include <ZarafaCode.h>

struct IStream;
class ECFifoBuffer;

#ifdef DEBUG
#define STR_DEF_TIMEOUT 0
#else
#define STR_DEF_TIMEOUT 600000
#endif

class ECSerializer
{
public:
	virtual ~ECSerializer() {};

	virtual ECRESULT SetBuffer(void *lpBuffer) = 0;

	virtual ECRESULT Write(const void *ptr, size_t size, size_t nmemb) = 0;
	virtual ECRESULT Read(void *ptr, size_t size, size_t nmemb) = 0;

	virtual ECRESULT Skip(size_t size, size_t nmemb) = 0;
	virtual ECRESULT Flush() = 0;
	
	virtual ECRESULT Stat(ULONG *lpulRead, ULONG *lpulWritten) = 0;
};

class ECStreamSerializer : public ECSerializer
{
public:
	ECStreamSerializer(IStream *lpBuffer);
	~ECStreamSerializer();

	ECRESULT SetBuffer(void *lpBuffer);

	ECRESULT Write(const void *ptr, size_t size, size_t nmemb);
	ECRESULT Read(void *ptr, size_t size, size_t nmemb);

	ECRESULT Skip(size_t size, size_t nmemb);
	ECRESULT Flush();
	
	ECRESULT Stat(ULONG *lpulRead, ULONG *lpulWritten);

private:
	IStream *m_lpBuffer;
	ULONG m_ulRead;
	ULONG m_ulWritten;
};

class ECFifoSerializer : public ECSerializer
{
public:
	enum eMode { serialize, deserialize };

	ECFifoSerializer(ECFifoBuffer *lpBuffer, eMode mode);
	~ECFifoSerializer();

	ECRESULT SetBuffer(void *lpBuffer);

	ECRESULT Write(const void *ptr, size_t size, size_t nmemb);
	ECRESULT Read(void *ptr, size_t size, size_t nmemb);

	ECRESULT Skip(size_t size, size_t nmemb);
	ECRESULT Flush();
	
	ECRESULT Stat(ULONG *lpulRead, ULONG *lpulWritten);

private:
	ECFifoBuffer *m_lpBuffer;
	eMode m_mode;
	ULONG m_ulRead;
	ULONG m_ulWritten;
};

#endif /* ECSERIALIZER_H */
