#ifndef ECTHREADUTIL_H
#define ECTHREADUTIL_H

#include <pthread.h>

class CPthreadMutex {
public:
	CPthreadMutex(bool bRecurse = false);
	~CPthreadMutex();

	void Lock();
	void Unlock();

private:
	pthread_mutex_t m_hMutex;
};


class scoped_lock {
public:
	scoped_lock(pthread_mutex_t &mutex) : m_mutex(mutex) {
		pthread_mutex_lock(&m_mutex);
	}

	~scoped_lock() { 
		pthread_mutex_unlock(&m_mutex);
	}

private:
	// Make sure an object is not accidentally copied
	scoped_lock(const scoped_lock &);
	scoped_lock& operator=(const scoped_lock &);

	pthread_mutex_t &m_mutex;
};

#define WITH_LOCK(_lock)	\
	for (scoped_lock __lock(_lock), *ptr = NULL; ptr == NULL; ptr = &__lock)


template<int(*fnlock)(pthread_rwlock_t*)>
class scoped_rwlock {
public:
	scoped_rwlock(pthread_rwlock_t &rwlock) : m_rwlock(rwlock) {
		fnlock(&m_rwlock);
	}

	~scoped_rwlock() { 
		pthread_rwlock_unlock(&m_rwlock);
	}

private:
	// Make sure an object is not accidentally copied
	scoped_rwlock(const scoped_rwlock &);
	scoped_rwlock& operator=(const scoped_rwlock &);

	pthread_rwlock_t &m_rwlock;
};

typedef scoped_rwlock<pthread_rwlock_wrlock> scoped_exclusive_rwlock;
typedef scoped_rwlock<pthread_rwlock_rdlock> scoped_shared_rwlock;

#endif //#ifndef ECTHREADUTIL_H
